@@ -1,11 +1,11 @@
-from __future__ import annotations
-
 import asyncio
-from concurrent.futures import Future
-from typing import Any, ClassVar, Final, List, NamedTuple, Optional, Set, Union
+import threading
+from threading import Event
+from typing import Any, ClassVar, Final, List, Optional, Set, Union
 
-from robotcode.core.async_tools import Event
+from robotcode.core.concurrent import Task
 from robotcode.core.event import event
+from robotcode.core.language import LanguageDefinition
 from robotcode.core.lsp.types import (
     CancelParams,
     ClientCapabilities,
@@ -41,7 +41,6 @@ from robotcode.jsonrpc2.protocol import (
 )
 from robotcode.jsonrpc2.server import JsonRPCServer
 
-from .has_extend_capabilities import HasExtendCapabilities
 from .parts.code_action import CodeActionProtocolPart
 from .parts.code_lens import CodeLensProtocolPart
 from .parts.commands import CommandsProtocolPart
@@ -59,6 +58,7 @@ from .parts.implementation import ImplementationProtocolPart
 from .parts.inlay_hint import InlayHintProtocolPart
 from .parts.inline_value import InlineValueProtocolPart
 from .parts.linked_editing_ranges import LinkedEditingRangeProtocolPart
+from .parts.protocol_part import LanguageServerProtocolPart
 from .parts.references import ReferencesProtocolPart
 from .parts.rename import RenameProtocolPart
 from .parts.selection_range import SelectionRangeProtocolPart
@@ -67,19 +67,13 @@ from .parts.signature_help import SignatureHelpProtocolPart
 from .parts.window import WindowProtocolPart
 from .parts.workspace import Workspace
 
-__all__ = ["LanguageServerException", "LanguageServerProtocol", "HasExtendCapabilities"]
+__all__ = ["LanguageServerException", "LanguageServerProtocol"]
 
 
 class LanguageServerException(JsonRPCException):
     pass
 
 
-class LanguageDefinition(NamedTuple):
-    id: str
-    extensions: List[str]
-    aliases: Optional[List[str]] = None
-
-
 class LanguageServerProtocol(JsonRPCProtocol):
     __logger = LoggingDescriptor()
 
@@ -109,13 +103,15 @@ class LanguageServerProtocol(JsonRPCProtocol):
 
     name: Optional[str] = None
     short_name: Optional[str] = None
+    progress_title: Optional[str] = None
     version: Optional[str] = None
 
     file_extensions: ClassVar[Set[str]] = set()
-    languages: ClassVar[List[LanguageDefinition]] = []
+    language_definitions: ClassVar[List[LanguageDefinition]] = []
 
     def __init__(self, server: JsonRPCServer[Any]):
         super().__init__()
+        self.running_thread = threading.current_thread()
         self.server = server
         self.parent_process_id: Optional[int] = None
         self.initialization_options: Any = None
@@ -172,7 +168,7 @@ class LanguageServerProtocol(JsonRPCProtocol):
         base_capabilities = replace(self._base_capabilities)
 
         for p in self.registry.parts:
-            if isinstance(p, HasExtendCapabilities):
+            if isinstance(p, LanguageServerProtocolPart):
                 p.extend_capabilities(base_capabilities)
 
         return base_capabilities
@@ -181,7 +177,10 @@ class LanguageServerProtocol(JsonRPCProtocol):
 
     def start_parent_process_watcher(self) -> None:
         if self.parent_process_id and self.loop:
-            self.loop.call_later(self.PARENT_PROCESS_WATCHER_INTERVAL, self._parent_process_watcher)
+            self.loop.call_later(
+                self.PARENT_PROCESS_WATCHER_INTERVAL,
+                self._parent_process_watcher,
+            )
 
     def _parent_process_watcher(self) -> None:
         if not self.parent_process_id:
@@ -216,10 +215,15 @@ class LanguageServerProtocol(JsonRPCProtocol):
 
         self.client_capabilities = capabilities
 
-        self._workspace = Workspace(self, root_uri=root_uri, root_path=root_path, workspace_folders=workspace_folders)
+        self._workspace = Workspace(
+            self,
+            root_uri=root_uri,
+            root_path=root_path,
+            workspace_folders=workspace_folders,
+        )
 
         folders = (
-            ", ".join((f"'{v.name}'" for v in self._workspace.workspace_folders))
+            ", ".join(f"'{v.name}'" for v in self._workspace.workspace_folders)
             if self._workspace.workspace_folders
             else ""
         )
@@ -243,7 +247,9 @@ class LanguageServerProtocol(JsonRPCProtocol):
                 raise
             except BaseException as e:
                 raise JsonRPCErrorException(
-                    JsonRPCErrors.INTERNAL_ERROR, f"Can't start language server: {e}", InitializeError(retry=False)
+                    JsonRPCErrors.INTERNAL_ERROR,
+                    f"Can't start language server: {e}",
+                    InitializeError(retry=False),
                 ) from e
 
             return InitializeResult(
@@ -261,12 +267,19 @@ class LanguageServerProtocol(JsonRPCProtocol):
     def on_initialize(sender, initialization_options: Optional[Any] = None) -> None:  # pragma: no cover, NOSONAR
         ...
 
-    @rpc_method(name="initialized", param_type=InitializedParams)
+    @rpc_method(name="initialized", param_type=InitializedParams, threaded=True)
     def _initialized(self, params: InitializedParams, *args: Any, **kwargs: Any) -> None:
+        if self.is_initialized.is_set():
+            return
+
         self.on_initialized(self)
 
         self.is_initialized.set()
 
+    def ensure_initialized(self) -> None:
+        if not self.is_initialized.wait(60):
+            raise RuntimeError("Language server not initialized")
+
     @event
     def on_initialized(sender) -> None:  # pragma: no cover, NOSONAR
         ...
@@ -281,7 +294,7 @@ class LanguageServerProtocol(JsonRPCProtocol):
 
         try:
             self.cancel_all_received_request()
-        except BaseException as e:  # NOSONAR
+        except BaseException as e:
             self.__logger.exception(e)
 
         self.on_shutdown(self)
@@ -303,22 +316,28 @@ class LanguageServerProtocol(JsonRPCProtocol):
     def _cancel_request(self, id: Union[int, str], **kwargs: Any) -> None:
         self.cancel_request(id)
 
-    def register_capability(self, id: str, method: str, register_options: Optional[Any]) -> Future[None]:
+    def register_capability(self, id: str, method: str, register_options: Optional[Any]) -> Task[None]:
         return self.register_capabilities([Registration(id=id, method=method, register_options=register_options)])
 
-    def register_capabilities(self, registrations: List[Registration]) -> Future[None]:
+    def register_capabilities(self, registrations: List[Registration]) -> Task[None]:
         if not registrations:
-            result: Future[None] = Future()
+            result: Task[None] = Task()
             result.set_result(None)
             return result
-        return self.send_request("client/registerCapability", RegistrationParams(registrations=registrations))
+        return self.send_request(
+            "client/registerCapability",
+            RegistrationParams(registrations=registrations),
+        )
 
-    def unregister_capability(self, id: str, method: str) -> Future[None]:
+    def unregister_capability(self, id: str, method: str) -> Task[None]:
         return self.unregister_capabilities([Unregistration(id=id, method=method)])
 
-    def unregister_capabilities(self, unregisterations: List[Unregistration]) -> Future[None]:
+    def unregister_capabilities(self, unregisterations: List[Unregistration]) -> Task[None]:
         if not unregisterations:
-            result: Future[None] = Future()
+            result: Task[None] = Task()
             result.set_result(None)
             return result
-        return self.send_request("client/unregisterCapability", UnregistrationParams(unregisterations=unregisterations))
+        return self.send_request(
+            "client/unregisterCapability",
+            UnregistrationParams(unregisterations=unregisterations),
+        )
