@@ -404,7 +404,7 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart):
             ):
                 if (
                     namespace.find_keyword(
-                        token.value,
+                        unescape(token.value),  # TODO: this must be resovle possible variables
                         raise_keyword_error=False,
                         handle_bdd_style=False,
                     )
@@ -458,7 +458,9 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart):
 
                 kw_namespace: Optional[str] = None
                 kw: str = token.value
-                kw_doc = namespace.find_keyword(token.value, raise_keyword_error=False)
+                kw_doc = namespace.find_keyword(
+                    unescape(token.value), raise_keyword_error=False
+                )  # TODO: this must be resovle possible variables
 
                 (
                     lib_entry,
@@ -491,7 +493,7 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart):
                     )
 
                 if builtin_library_doc is not None and kw in builtin_library_doc.keywords:
-                    if kw_doc is not None and kw_doc.libname == cls.BUILTIN_MATCHER and kw_doc.matcher == kw:
+                    if kw_doc is not None and kw_doc.libname == cls.BUILTIN_MATCHER and kw_doc.matcher.match_string(kw):
                         if not sem_mod:
                             sem_mod = set()
                         sem_mod.add(RobotSemTokenModifiers.BUILTIN)
@@ -618,8 +620,7 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart):
                 if (
                     yield_arguments
                     or token.type != Token.ARGUMENT
-                    or token.type != Token.NAME
-                    and cached_isinstance(node, Metadata)
+                    or (token.type != Token.NAME and cached_isinstance(node, Metadata))
                 ):
                     yield SemTokenInfo.from_token(token, sem_type, sem_mod, col_offset, length)
 
@@ -630,10 +631,8 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart):
         namespace: Namespace,
         builtin_library_doc: Optional[LibraryDoc],
     ) -> Iterator[SemTokenInfo]:
-        if (
-            token.type in {Token.ARGUMENT, Token.TESTCASE_NAME, Token.KEYWORD_NAME}
-            or token.type == Token.NAME
-            and cached_isinstance(node, VariablesImport, LibraryImport, ResourceImport)
+        if token.type in {Token.ARGUMENT, Token.TESTCASE_NAME, Token.KEYWORD_NAME} or (
+            token.type == Token.NAME and cached_isinstance(node, VariablesImport, LibraryImport, ResourceImport)
         ):
             if (
                 cached_isinstance(node, Variable) and token.type == Token.ARGUMENT and node.name and node.name[0] == "&"
@@ -1121,14 +1120,14 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart):
                             kw: Optional[str] = None
 
                             for _, n in iter_over_keyword_names_and_owners(
-                                ModelHelper.strip_bdd_prefix(namespace, kw_token).value
+                                unescape(ModelHelper.strip_bdd_prefix(namespace, kw_token).value)
                             ):
                                 if n is not None:
                                     matcher = KeywordMatcher(n)
                                     if matcher in ALL_RUN_KEYWORDS_MATCHERS:
                                         kw = n
                             if kw:
-                                kw_doc = namespace.find_keyword(kw_token.value)
+                                kw_doc = namespace.find_keyword(unescape(kw_token.value))
                                 if kw_doc is not None and kw_doc.is_any_run_keyword():
                                     for kw_res in self.generate_run_kw_tokens(
                                         namespace,
